import { T, cond, pipe, always, not, is, contains, __ } from 'ramda'

const operations = ['add', 'replace', 'test', 'remove', 'move', 'copy']

/**
 * Tests if a given string is a valid JSON Patch operation name
 * 
 * ** Sig: String -> Boolean|String **
 *
 * @function
 * @param {string} operation - the operation to be tested
 * @returns {true|string} result - ** true ** if the operation is valid or a ** string ** with the corresponding error
 * @example
 *
 * import isOperation from 'json-patch-utils'
 * isOperation('add') // true
 * isOperation('foobar') // "The operation name is not among the valid names"
 *
 */
const isOperation = cond([
  [
    pipe(is(String), not),
    always('The operation property must be a string')
  ],
  [
    pipe(contains(__, operations), not),
    always(`The operation name is not among the valid names ${operations.join(', ')}`)
  ],
  [T, T]
])

export default isOperation
